import itertools, random

base_notes = ['C', 'C#', 'D', 'D#', 'E', 'F', 'F#', 'G', 'G#', 'A', 'A#', 'B']
notes_list = [x + y for y in  [str(i) for i in range(0, 9)] for x in base_notes]
pitch_offset = 24

keys_diffs = {
    'major' : [0, 2, 4, 5, 7, 9, 11],
    'minor' : [0, 2, 3, 5, 7, 8, 10],
    'acoustic' : [0, 2, 4, 6, 7, 9, 10],
    'algerian' : [0, 2, 3, 6, 7, 8, 11, 12, 14, 15, 17],
    'altered' : [0, 1, 3, 4, 6, 8, 10], 
    'augmented' : [0, 3, 4, 7, 8, 11],
    'bebop' : [0, 2, 4, 5, 7, 9, 10, 11],
    'blues' : [0, 3, 5, 6, 7, 10],
    'chromatic' : [0, 1, 2, 3, 4, 5,6, 7, 8, 9, 10, 11],
    'double_harmonic' : [0, 1, 4, 5, 7, 8, 11],
    'enigmatic' : [0, 1, 5, 6, 8, 9, 11],
    'flamenco' : [0, 1, 4, 5, 7, 8, 11], 
    'gypsy' : [0, 2, 3, 6, 7, 8, 10], 
    'half_iminished' : [0, 2, 3, 5, 6, 8, 10],
    'harmonic_major' : [0, 2, 4, 5, 7, 8, 11],
    'harmonic_minor' : [0, 2, 3, 5, 7, 8, 11],
    'hijaroshi' : [0, 4, 6, 7, 11],
    'hungarian_minor' : [0, 2, 3, 6, 7, 8, 11],
    'insen' : [0, 1, 5, 7, 10],
    'iwato' : [0, 1, 5, 6, 10], 
    'locrian' : [0, 1, 3, 5, 6, 8, 10],
}

def get_pitch(note):
    return notes_list.index(note) + pitch_offset

#Note class; mostly used to add data to a midi file. 
class Note:
    def __init__(self, track = 0, channel = 0, note = 'A0', time = 0, duration = 0, volume = 0):
        self.track = track
        self.note = note
        self.channel = channel - 1 #in code, channels are numbered 0-15, while in files they're 1-16. This matters when using percussion (channel 9 in code, 10 in files). 
        self.pitch = get_pitch(note)
        self.time = time
        self.duration = duration
        self.volume = volume

    def get_values(self):
        return self.track, self.channel, self.pitch, self.time, self.duration, self.volume
        

#The key class offers functionality relating to generating random notes based off of some rules. 
#The key argument is what the root note of the key is, for instance, the note A. This argument can be ommited if using the base_notes argument. 
#The scale is the name of the scale that the other notes will be generated off. For instance, if using a minor scale with the A as the root of the key, if will generate all the notes belonging to the A minor scale. This argument can be ommited if using the base_notes argument. 
#If you wish to define your own notes that define the key (maybe I missed one or you just want to use a group of notes), you can supply a list of base notes that the key will use (example : ['A', 'C', 'D', 'D#', G#']). This list should not contain flat notes (yes, I know there's a reason for flat and sharp notes existing, but for the sake of code simplicity, I just use sharp notes). 

class Key:
    
    def __init__(self, root_note = 'A', scale = 'minor', b_notes = [], notes_bias = {}, low_end = 'A2', high_end = 'G#6', markov_values = None):
        self.scale = scale
        self.keys_diffs = keys_diffs
        self.root_note = root_note
        root_note_index = base_notes.index(root_note)
        self.diffs = [(x + root_note_index)%len(base_notes) for x in self.keys_diffs[scale]]
        notes_bias = {str((int(x) + root_note_index)%len(base_notes)) : notes_bias[x] for x in notes_bias}
        
        for note_diff in notes_bias:  
            if int(note_diff) in self.diffs:
                self.diffs += [int(note_diff) for i in range(notes_bias[note_diff])]

        if not b_notes : 
            self.base_notes = [base_notes[x] for x in self.diffs]
        else : 
            self.base_notes = b_notes
      
        self.notes = [[x] * self.base_notes.count(x[:-1]) for x in notes_list if x[:-1] in self.base_notes and notes_list.index(low_end) <= notes_list.index(x) <= notes_list.index(high_end)]
        self.notes = [item for sublist in self.notes for item in sublist]

        #Markov values are used for generating notes using Markov chains.  
        #It's explained a bit better below, at the function. 
        self.markov_values = markov_values
        if markov_values: 
            depth = lambda L: isinstance(L, list) and max(map(depth, L)) +1 
            markov_depth = depth(markov_values)
            self.markov_prev_notes = [self.notes.index(self.generate_note(None, 3)) for i in range(markov_depth-1)]



    #This function generates a note regarding the note_pivot it is given. 
    #A pivot is required so the notes chosen aren't just completely arbitrary, and so with a given pivot and radius, the generator will pick notes relatively close to each other. 
    #The bias_same_note is an int argument <100 that is a probability for the generator to choose the same note as the pivot. If using the note_timing module, notes with the same value are squished together, which is basically how you get notes of varying length. 
    def generate_note(self, note_pivot = None, note_radius = 3, bias_same_note = 0, dir_write_note = ''):
        if random.randint(0, 100) < bias_same_note and note_pivot: 
            return note_pivot
            
        if not note_pivot: 
            return random.choice(self.notes)
        else:
            if self.markov_values:
                note = self.generate_note_markov(note_pivot)
                note_index = None
            else: 
                pivot_index = self.notes.index(note_pivot)
                note_index = random.randint(max(0, pivot_index - note_radius), min(pivot_index + note_radius, len(self.notes)-1))
                note = self.notes[note_index]
           
        if dir_write_note: 
            with open(dir_write_note, 'a+') as f: 
                if not note_index: 
                    note_index = self.notes.index(note)
                f.write(', ' + str(note_index % 16))
        return note


    #This function generates a note but by using markov chain principles instead. Basically, there is (or should be) a file containing values that help choose the note based on the previous one. The values are contained in a list like so : 
    #markov_values = [[0.2, 0.1, 0.2, 0.05, ...], [0.1, 0.2, ...]] 
    #Here, each list represents the values for a note (for instance, the 0th list represents the values for the 0th note in the scale), and each value in that list is the probability that that is the chosen note from the chain. More on this in the readme. 
    #As of a recent update, you can have multiple layers in the chains. 

    def generate_note_markov(self, prev_note):
        if not self.markov_values: 
            print 'No markov values found, but tried to generate using Markov chains. '
            print 'Ensure you are not using the Markov attribute, or include a markov_file and initialize the Key object with it. '
            exit()
        prev_notes = self.markov_prev_notes[1:] + [self.notes.index(prev_note)]
        note_index = self.base_notes.index(prev_note[:-1])
        note_table = self.markov_values

        for note in prev_notes: 
            note_table = note_table[note]

        note_table = [x/sum(note_table) for x in note_table]
        r, s = random.random(), 0
        for i in range(len(note_table)):
            note = note_table[i]
            s+= note
            if s >= r: 
                note_index = i % len(self.notes)
                return self.notes[note_index]
        #If we get here, something wrong happened because at some point, s has to be greater than r. 
        print 'Something went wrong at generating markov note, random int was ', r, ' and sum was ', s
        exit()

#A bar is a class containing several notes. The class is mainly used to accent notes. 
#The accent_notes function receives a dictionary as an argument, but the user can manually set the accents property. 
#The accents is a dictionary that looks like {1 : 100, 3 : 80}, where the key is the note that is accented, and the value is the volume. This accenting dictionary will make the first note with the highest volume, and the 3rd with 80% of the highest volume. 
#All other notes use the default value used in the constructor. 
class Bar:
    def __init__(self, notes, accents = {}, default_accent = 50):
        self.notes = notes
        for note in self.notes : note.volume = default_accent
        self.accents = accents
        if accents : self.accent_notes(accents)
        
    def accent_notes(self, accents = {}):
        if not self.accents : self.accents = accents
        for accent in accents:
            try:
                self.notes[accent].volume = accents[accent]
            except Exception:
                pass
        
def main():
    a = Key('B', 'minor')
    print a.notes
    
if __name__ == '__main__' : main()



import sys
from ninopianino import generator, template_utils

def generate_blocks():
    blocks = []
    for i in range(1, 90):
        new_block = {
            'program_number' : i, 
            'play_at' : [(i-1) * 8], 
            'number_of_bars' : 2, 
            'number_of_beats_per_bar' : 4,
        }
        blocks.append(new_block)
        print 'Program number ', i, 'between ', (i-1)*4, 'and', i*4
    return blocks

def main():
    base_block = template_utils.create_base_block()
    base_block['channel'] = 1
    base_block['root_note'] = 'C'
    base_block['scale'] = 'major'
    base_block['blocks'] = generate_blocks()
    base_block['low_end'] = 'C2'
    base_block['high_end'] = 'C3'
    mid = generator.generate(base_block)
    try: 
        soundfont = sys.argv[1]
    except: 
        import traceback
        traceback.print_exc()
        soundfont = 'soundfonts/FluidR3_GM.sf2' 
    print 'Testing ', soundfont
    song_path = generator.write_mid(mid, 'soundfont_test', soundfont )


main()



from setuptools import setup
import os
import subprocess
import argparse

def main():
    required_packages = ['fluidsynth', 'lame']
    for package in required_packages: 
        subprocess.call(['apt-get', 'install', package])

    if os.environ.get('GET_SOUNDFONT'): 
        subprocess.call(['mkdir', 'soundfonts'])
        subprocess.call(['wget', 'ftp://sourceforge.nchc.org.tw/a/an/androidframe/soundfonts/FluidR3_GM.sf2'])
        subprocess.call(['mv', 'FluidR3_GM.sf2', 'soundfonts'])

    setup(
        name = 'ninopianino',
        packages = ['ninopianino'],
        version = '0.1.7', 
        description = 'Programmable music generator. ', 
        author = 'Nikola Dokoski', 
        author_email = 'ninodokoskiot@hotmail.com', 
        url = 'https://github.com/NinoDoko/nino_pianino', 
        download_url = 'https://github.com/NinoDoko/nino_pianino/tarball/0.1',
        scripts = ['ninopianino/pianize'],
        install_requires = [
            'pyhaikunator',
            'midiutil',
            ]
        )


main()



import random
from music_models import base_notes, keys_diffs, Key
from generator import gen_notes_for_key

chord_progressions = [
    [(0, 'major'), (5, 'major'), (7, 'major')], # I-IV-V (C-F-G)
    [(0, 'major'), (5, 'major'), (2, 'major')], # I-V-II (C-G-D)
    [(0, 'major'), (9, 'minor'), (7, 'major'), (2, 'minor'), (2, 'major')], #I-vi-VII-ii-II (C-Am-G-Dm-D)
    [(0, 'major'), (9, 'minor'), (2, 'minor'), (7, 'major')], #I-vi-ii-V (C-Am-Dm-G)
    [(0, 'major'), (7, 'major'), (2, 'minor'), (9, 'minor'), (5, 'major'), (5, 'minor')], #I-V-ii-vi-IV-iv (C-G-Dm-Am-F-Fm)
]

def generate_pattern(block, min_note_len = 1, max_note_len = 4):
    pattern_sum = block['number_of_beats_per_bar']
    pattern = []
    while sum(pattern) < pattern_sum:
        pattern.append(min(random.randint(min_note_len, max_note_len), pattern_sum - sum(pattern)))
    return pattern

def index_progression_to_notes(root_note, progression):
    return [(base_notes[(root_note + chord[0])%len(base_notes)], chord[1]) for chord in progression]
    

#You can pass root_note as either the index or the value of the note. 
def generate_chord_progression(root_note):
    progression = random.choice(chord_progressions)
    if root_note in base_notes: 
        root_note = base_notes.index(root_note)
    #Here we assume root_note is the index of the root note. 
#    progression = [(base_notes[(root_note + chord[0])%len(base_notes)], chord[1]) for chord in progression]
    progression = index_progression_to_notes(root_note, progression)
    return progression



def generate_random_chord_progression(root_note, scale, number_of_chords, scale_choices = ['major', 'minor'], markov_values = None, exp_var = 4.0):
    root_key = Key(root_note = root_note, scale = scale)
    root_key_base = root_key.base_notes

    chord_notes = [x.note[:-1] for x in gen_notes_for_key(1, number_of_chords, root_note, scale, 1, markov_values)]
    progression = []

    all_keys = [Key(note, scale) for note in root_key.base_notes for scale in scale_choices]

    keys_candidates_ctrs = [(key, 0.0 + len([x for x in key.base_notes if x in root_key.base_notes]) ** exp_var) for key in all_keys]

    key_candidates_probabilities = [(x[0], x[1]/sum([x[1] for x in keys_candidates_ctrs])) for x in keys_candidates_ctrs]
    if sum(x[1] for x in key_candidates_probabilities) > 1 :
        print 'The sum is : ', sum(x[1] for x in key_candidates_probabilities)

    while len(progression) < number_of_chords: 
        r, s = random.random(), 0
        for key_prob in key_candidates_probabilities:
            s += key_prob[1]
            if s >= r: 
                progression.append((key_prob[0].root_note, key_prob[0].scale))
                break
    return progression
           
        
           

def create_base_block(bpm = 120, play_at = [0], name = 'base', track = 1):
    return {'bpm' : bpm, 'play_at' : play_at, 'name' : name, 'block_type' : 'complex', 'track' : track, 'blocks' : []}


def create_percussion(block, no_hits = 1, no_cymbals = 1):
    percussion_base = {"block_type":"complex", "name":"percussions",  "track":10, "channel":10, "default_accent":80, "play_at":[0], "number_of_beats_per_bar" : block["number_of_beats_per_bar"], "number_of_bars" : block["number_of_bars"], "blocks": []} 
    for i in range(no_hits):
        percussion_base['blocks'].append({
            "name": "percussions_" + str(i),
            "play_at": [0],
            "root_note": "C",
            "scale": "major",
            "low_end": "B0",
            "high_end": "G1",
        })
        no_hits -= 1
    for i in range(no_cymbals):
        percussion_base['blocks'].append({
            "name": "percussions_cymbals_" + str(i),
            "play_at": [0],
            "base_notes" : ['F#', 'F#', 'F#', 'F#', 'F#', 'G#', 'G#', 'G#', 'G#', 'A#', 'A#', 'C#', 'D#'],
            "low_end": "F#1",
            "high_end": "D#2",
        })
        no_hits -= 1
    return percussion_base


#chords is a list of (root_note, scale) pairs. For instance : [('A', 'minor'), ('C', 'major'), ('E', 'minor')]
def create_chord_progression(block, chords = [], extra_kwargs = {}):
    no_beats = block['number_of_beats_per_bar']
    no_bars = block['number_of_bars'] / len(chords)
    blocks = []
    for i in range(len(chords)): 
        new_block = {
            'name' : 'gen_chord_' + chords[i][0], 
            'play_at' : [i * no_beats * no_bars],
            'root_note' : chords[i][0], 
            'scale' : chords[i][1], 
            'number_of_bars' : no_bars,
            'number_of_beats_per_bar' : block['number_of_beats_per_bar']
        }
        for kwarg in extra_kwargs: 
            new_block[kwarg] = extra_kwargs[kwarg]
        blocks.append(new_block)
    return blocks

def repeat_block(block, number_repeats, number_of_bars):
    play_at = block['play_at']
    result = []
    for entry in play_at:
        result += [entry] + [entry + i * block.get('number_of_beats_per_bar', 1) * number_of_bars for i in range(1, number_repeats)]
    return result

#from sample_markov import markov_values
#generate_random_chord_progression('C', 'major', 5, markov_values = markov_values)




import music_models, random



#Time_signature should be a string formatted as x/y, for instance 3/8. 
#The function makes a list of bars (where a bar is a list of notes) and in those bars are tuples of notes + length of note in beats. 
def group_notes_for_time_signature(notes, no_beats, bias_separate_notes = 0, accents = {}, start_at = 0, pattern = [], time_signature = '', default_accent = 50):
    notes = [notes[i:i+no_beats] for i in range(0, len(notes), no_beats)]
    
    result = [[]]
    t = start_at
    for bar in notes:
        new_bar = []
        i = 0
        if pattern: 
            for i in range(len(bar)-1): 
                try:
                    bar[i].duration = pattern[i]
                    bar[i].time = t
                    t += pattern[i]
                    new_bar.append(bar[i])
                except Exception: 
                    continue
        else: 
            while i < no_beats :
                if len(bar) < no_beats : break
                length = 0.0
                j = i
                while bar[i].pitch == bar[j].pitch :
                    length += 1
                    j += 1
                    if random.randint(0, 100) < bias_separate_notes or j == no_beats : 
                        break
                bar[i].duration = length
                if length > 20:
                    print 'Length exceeded!'
                    print length
                    exit()
                bar[i].time = t
                t += length
                new_bar.append(bar[i])
                i = j
        result.append(music_models.Bar(new_bar, accents, default_accent))
    return [x for x in result if x]
